# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import (Plugin, IndependentPlugin, SoSPredicate)


class FirewallTables(Plugin, IndependentPlugin):
    """Collects information about local firewall tables, such as iptables,
    and nf_tables (via nft). Note that this plugin does _not_ collect firewalld
    information, which is handled by a separate plugin.

    Collections from this plugin are largely gated byt the presence of relevant
    kernel modules - for example,  the plugin will not collect the nf_tables
    ruleset if both the `nf_tables` and `nfnetlink` kernel modules are not
    currently loaded (unless using the --allow-system-changes option).
    """

    short_desc = 'firewall tables'

    plugin_name = "firewall_tables"
    profiles = ('network', 'system')
    files = ('/etc/nftables',)
    kernel_mods = ('ip_tables', 'ip6_tables', 'nf_tables', 'nfnetlink',
                   'ebtables')

    def collect_iptable(self, tablename):
        """ Collecting iptables rules for a table loads either kernel module
        of the table name (for kernel <= 3), or nf_tables (for kernel >= 4).
        If neither module is present, the rules must be empty."""

        modname = "iptable_" + tablename
        cmd = "iptables -t " + tablename + " -nvL"
        self.add_cmd_output(
            cmd,
            pred=SoSPredicate(self, kmods=[modname, 'nf_tables']))

    def collect_ip6table(self, tablename):
        """ Same as function above, but for ipv6 """

        modname = "ip6table_" + tablename
        cmd = "ip6tables -t " + tablename + " -nvL"
        self.add_cmd_output(
            cmd,
            pred=SoSPredicate(self, kmods=[modname, 'nf_tables']))

    def collect_nftables(self):
        """ Collects nftables rulesets with 'nft' commands if the modules
        are present """

        # collect nftables ruleset
        nft_pred = SoSPredicate(self,
                                kmods=['nf_tables', 'nfnetlink'],
                                required={'kmods': 'all'})
        return self.collect_cmd_output("nft -a list ruleset", pred=nft_pred,
                                       changes=True)

    def setup(self):
        # first, collect "nft list ruleset" as collecting commands like
        # ip6tables -t mangle -nvL
        # depends on its output
        # store in nft_ip_tables lists of ip[|6] tables from nft list
        nft_list = self.collect_nftables()
        nft_ip_tables = {'ip': [], 'ip6': []}
        nft_lines = nft_list['output'] if nft_list['status'] == 0 else ''
        for line in nft_lines.splitlines():
            words = line.split()[0:3]
            if len(words) == 3 and words[0] == 'table' and \
                    words[1] in nft_ip_tables:
                nft_ip_tables[words[1]].append(words[2])
        # collect iptables -t for any existing table, if we can't read the
        # tables, collect 2 default ones (mangle, filter)
        # do collect them only when relevant nft list ruleset exists
        default_ip_tables = "mangle\nfilter\nnat\n"
        try:
            proc_net_ip_tables = '/proc/net/ip_tables_names'
            with open(proc_net_ip_tables, 'r', encoding='UTF-8') as ifile:
                ip_tables_names = ifile.read()
        except IOError:
            ip_tables_names = default_ip_tables
        for table in ip_tables_names.splitlines():
            if nft_list['status'] == 0 and table in nft_ip_tables['ip']:
                self.collect_iptable(table)
        # collect the same for ip6tables
        try:
            proc_net_ip6_tables = '/proc/net/ip6_tables_names'
            with open(proc_net_ip6_tables, 'r', encoding='UTF-8') as ipfile:
                ip_tables_names = ipfile.read()
        except IOError:
            ip_tables_names = default_ip_tables
        for table in ip_tables_names.splitlines():
            if nft_list['status'] == 0 and table in nft_ip_tables['ip6']:
                self.collect_ip6table(table)

        # When iptables is called it will load:
        # 1) the modules iptables_filter (for kernel <= 3) or
        #    nf_tables (for kernel >= 4) if they are not loaded.
        # 2) nft 'ip filter' table will be created
        # The same goes for ipv6.
        if nft_list['status'] != 0 or 'filter' in nft_ip_tables['ip']:
            self.add_cmd_output(
                "iptables -vnxL",
                pred=SoSPredicate(self, kmods=['iptable_filter', 'nf_tables'])
            )
        if nft_list['status'] != 0 or 'filter' in nft_ip_tables['ip6']:
            self.add_cmd_output(
                "ip6tables -vnxL",
                pred=SoSPredicate(self, kmods=['ip6table_filter', 'nf_tables'])
            )

        self.add_copy_spec([
            "/etc/nftables",
            "/etc/sysconfig/nftables.conf",
            "/etc/nftables.conf",
        ])

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class Pam(Plugin):

    short_desc = 'Pluggable Authentication Modules'

    plugin_name = "pam"
    profiles = ('security', 'identity', 'system')
    verify_packages = ('pam_.*',)
    security_libs = ""

    def setup(self):

        self.add_file_tags({
            '/etc/pam.d/password-auth': 'password_auth',
            '/etc/security/limits.*.conf': 'limits_conf'
        })

        self.add_copy_spec([
            "/etc/pam.d",
            "/etc/security",
            '/etc/authselect/authselect.conf',
        ])
        self.add_cmd_output([
            "pam_tally2",
            "faillock"
        ])

        self.add_dir_listing(self.security_libs)


class RedHatPam(Pam, RedHatPlugin):
    security_libs = "/lib*/security"

    def setup(self):
        super().setup()
        self.add_cmd_output(["authselect current"])


class DebianPam(Pam, DebianPlugin, UbuntuPlugin):
    security_libs = "/lib/x86_64-linux-gnu/security"


# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin


class StorageConsole(Plugin, RedHatPlugin, DebianPlugin):

    short_desc = 'Red Hat Storage Console'

    plugin_name = 'storageconsole'
    profiles = ('storage',)

    packages = ('rhscon-core',)

    def setup(self):
        all_logs = self.get_option("all_logs")

        if not all_logs:
            self.add_copy_spec([
                "/var/log/skyring/skyring.log",
                "/var/log/skyring/bigfin.log",
                "/var/log/carbon/console.log",
                "/var/log/graphite-web/info.log",
                "/var/log/graphite-web/exception.log",
            ])
        else:
            self.add_copy_spec([
                "/var/log/skyring/",
                "/var/log/carbon/",
                "/var/log/graphite-web/"
            ])

        self.add_copy_spec([
            "/etc/skyring/",
            "/etc/carbon/",
            "/etc/graphite-web/"
        ])

        self.add_cmd_output(
            "mongo skyring --eval 'db.getCollectionNames()'",
            suggest_filename="mongo_skyring_collectionnames.txt")
        self.add_cmd_output(
            "mongo skyring --eval 'DBQuery.shellBatchSize = 10000;"
            "db.storage_nodes.find()'",
            suggest_filename="mongo_skyring_storagenodes.txt")

# vim: set et ts=4 sw=4 :



# megacli.py
# Copyright (C) 2007-2014 Red Hat, Inc., Jon Magrini <jmagrini@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class MegaCLI(Plugin, RedHatPlugin):

    short_desc = 'LSI MegaRAID devices'

    plugin_name = 'megacli'
    profiles = ('system', 'storage', 'hardware')
    files = ('/opt/MegaRAID/MegaCli/MegaCli64',)

    def setup(self):
        cmd = '/opt/MegaRAID/MegaCli/MegaCli64'
        subcmds = [
            'LDPDInfo',
            '-AdpAllInfo',
            '-AdpBbuCmd -GetBbuStatus',
            '-ShowSummary'
        ]

        self.add_cmd_output([
            f"{cmd} {subcmd} -aALL" for subcmd in subcmds
        ])

# vim: set et ts=4 sw=4 :



# Copyright (C) 2018 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Date(Plugin, IndependentPlugin):

    short_desc = 'Basic system time information'

    plugin_name = 'date'

    def setup(self):

        self.add_cmd_output([
            'date',
            'date --utc',
            'hwclock'
        ], cmd_as_tag=True)

        self.add_copy_spec([
            '/etc/localtime',
            '/etc/adjtime',
        ])

# vim: set et ts=4 sw=4 :



# Copyright (C) 2024 ORNESS/Ditrit Drien Breton <drien.breton@orness.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.
import json

from sos.report.plugins import Plugin, DebianPlugin, PluginOpt


class Proxmox(Plugin, DebianPlugin):
    """
    This plugin will capture information about the system's
    Proxmox Virtualization Environment.

    It will collect information about the cluster, nodes, pools and
    storage from the Proxmox API.
    """
    short_desc = 'Proxmox cluster information'

    plugin_name = 'proxmox'
    packages = ('proxmox-ve',)

    option_list = [
        PluginOpt('output-formats',
                  desc='List of output formats to use '
                       'for the commands separated by ":".',
                  default='text',
                  ),
    ]

    def setup(self):
        output_formats = self.get_option('output-formats').split(':')

        commands = [
            'cluster/resources',
            'cluster/config/nodes',
            'cluster/options',
            'nodes/:id/status',
            'nodes/:id/storage',
            'nodes/:id/network',
            'pools',
            'storage',
            'storage/:sid',
            'cluster/ceph/status',
            'cluster/ceph/metadata',
            'cluster/ceph/flags',
        ]

        cmd_paths = []

        for command in commands:
            cmd_paths.extend(self.build_cmd_paths(command))

        self.add_cmd_output([
            f"pvesh get {cmd} {'--noborder' if format == 'text' else ''} "
            f"--output-format {format}"
            for cmd in cmd_paths
            for format in output_formats
        ])

    def build_cmd_paths(self, base_path):
        """
        Build command paths,
        replacing dynamic attributes with data from the API

        :param base_path: The base path to build from
        :type base_path: str

        :return: A list of paths
        """
        parts = base_path.split('/')
        paths = []

        id_index = next((i for i, part in enumerate(parts)
                         if part.startswith(':')), None)

        if id_index is not None:
            path = "/".join(parts[:id_index])
            trailing_path = "/".join(parts[id_index + 1:])

            results = self.exec_cmd(f'pvesh ls {path} --output-format json')

            if results["status"] == 0:
                children = json.loads(results["output"])
                for child in children:
                    paths.extend(
                        self.build_cmd_paths(
                            f"{path}/{child['name']}/{trailing_path}"
                        )
                    )

        else:
            paths.append('/'.join(parts))
        return paths

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, UbuntuPlugin


class CanonicaLivepatchOnprem(Plugin, UbuntuPlugin):

    short_desc = 'Canonical Livepatch Onprem Service'

    plugin_name = 'canonical_livepatch_onprem'
    profiles = ('services',)
    services = ("livepatch-server",)

    def setup(self):
        self.add_copy_spec([
            "/etc/livepatchd.yaml",
        ])

    def postproc(self):
        onprem_conf = "/etc/livepatchd.yaml"
        protect_keys = [
            "username",
            "password",
            "token",
            "connection_string",
        ]

        # Redact simple yaml style "key: value".
        keys_regex = fr"(^(-|\s)*({'|'.join(protect_keys)})\s*:\s*)(.*)"
        sub_regex = r"\1*********"
        self.do_path_regex_sub(onprem_conf, keys_regex, sub_regex)

        # Redact conf
        self.do_file_private_sub(onprem_conf)

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, UbuntuPlugin


class SunbeamHypervisor(Plugin, UbuntuPlugin):

    short_desc = "Sunbeam Hypervisor"

    plugin_name = "sunbeam_hypervisor"
    profiles = ('cloud',)
    packages = ('openstack-hypervisor',)

    plugin_timeout = 600

    services = (
        'snap.openstack-hypervisor.ceilometer-compute-agent',
        'snap.openstack-hypervisor.libvirt-exporter',
        'snap.openstack-hypervisor.libvirtd',
        'snap.openstack-hypervisor.masakari-instancemonitor',
        'snap.openstack-hypervisor.neutron-ovn-metadata-agent',
        'snap.openstack-hypervisor.nova-api-metadata',
        'snap.openstack-hypervisor.nova-compute',
        'snap.openstack-hypervisor.ovn-controller',
        'snap.openstack-hypervisor.ovs-exporter',
        'snap.openstack-hypervisor.ovs-vswitchd',
        'snap.openstack-hypervisor.ovsdb-server',
        'snap.openstack-hypervisor.virtlogd',
    )

    common_dir = '/var/snap/openstack-hypervisor/common'

    def setup(self):

        self.add_copy_spec([
            f'{self.common_dir}/*.log',
            f'{self.common_dir}/log/**/*.log',
            f'{self.common_dir}/etc',
            f'{self.common_dir}/lib/nova/instances/*/console.log',
            f'{self.common_dir}/cache/libvirt/qemu/capabilities/*.xml',
        ])

        self.add_forbidden_path([
            f'{self.common_dir}/etc/ssl/',
            f'{self.common_dir}/etc/libvirt/secrets',
            f'{self.common_dir}/etc/libvirt/passwd.db',
            f'{self.common_dir}/etc/libvirt/krb5.tab',
            f'{self.common_dir}/var/log/ovn/',
            f'{self.common_dir}/etc/pki/',
        ])

    def postproc(self):

        # libvirt confs
        match_exp = r"(\s*passwd=\s*')([^']*)('.*)"
        libvirt_path_exps = [
            fr"{self.common_dir}/etc/libvirt/qemu/.*\.xml",
            fr"{self.common_dir}/etc/libvirt/.*\.conf"
        ]
        for path_exp in libvirt_path_exps:
            self.do_path_regex_sub(path_exp, match_exp, r"\1******\3")

        # nova/neutron bits
        protect_keys = [
            ".*_key",
            ".*_pass(wd|word)?",
            "metadata_proxy_shared_secret",
            "password",
            "rbd_secret_uuid",
            "server_auth",
            "serverauth",
            "transport_url",
        ]
        connection_keys = ["connection", "sql_connection"]

        openstack_folders = [
            "nova",
            "neutron",
            "ceilometer",
            "masakarimonitors",
        ]

        self.do_path_regex_sub(
            fr'{self.common_dir}/etc/({"|".join(openstack_folders)})/*',
            fr'(^\s*({"|".join(protect_keys)})\s*=\s*)(.*)',
            r"\1*********"
        )
        self.do_path_regex_sub(
            fr'{self.common_dir}/etc/({"|".join(openstack_folders)})/*',
            fr'(^\s*({"|".join(connection_keys)})\s*=\s*(.*)'
            r'://(\w*):)(.*)(@(.*))',
            r"\1*********\6"
        )

        # hooks.log
        protect_hook_keys = [
            "password",
            "ovn_metadata_proxy_shared_secret",
            "cacert",
            "cert",
            "key",
            "ovn_cacert",
            "ovn_cert",
            "ovn_key",
            "url",
        ]

        self.do_file_sub(
            f'{self.common_dir}/hooks.log',
            fr'(\'({"|".join(protect_hook_keys)})\'):\s?\'(.+?)\'',
            r"\1: **********"
        )

# vim: et ts=4 sw=4



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import re
from sos.report.plugins import Plugin, RedHatPlugin


class Lustre(Plugin, RedHatPlugin):

    short_desc = 'Lustre filesystem'

    plugin_name = 'lustre'
    profiles = ('storage', 'network', 'cluster', )
    packages = ('lustre', 'lustre-client', )

    def get_params(self, name, param_list):
        """Use lctl get_param to collect a selection of parameters into a
            file.

        """
        self.add_cmd_output(f"lctl get_param {' '.join(param_list)}",
                            suggest_filename=f"params-{name}",
                            stderr=False)

    def setup(self):
        self.add_cmd_output([
            "lctl debug_kernel",
            "lctl device_list",
            "lctl list_nids",
            "lctl route_list",
            "lnetctl net show -v 4"
        ])

        # Grab almost everything
        output = self.exec_cmd("lctl list_param -R *")['output']
        exclude = (".*@.*|.*dump_page_cache|peers|.*quota.*|osd-.*|.*osc.*|"
                   "mgs.*|.*mgc_irstate|ldlm.*state|.*job_stats|.*exports.*")
        params = [item for item in output.splitlines()
                  if not re.match(exclude, item)]
        self.get_params("all", params)

        self.get_params(
            "basic",
            ["version", "health_check", "debug"]
        )

        # Client Specific
        self.add_cmd_output([
            "lfs df",
            "lfs df -i"
        ])
        self.get_params("osc_client", [
            "osc.*.max_dirty_mb",
            "osc.*.max_pages_per_rpc",
            "osc.*.checksums",
            "osc.*.max_rpcs_in_flight"
        ])

        # Server Specific
        self.get_params("osd", [
            "osd-*.*.{blocksize,brw_stats,enable_projid_xattr,extent*}",
            "osd-*.*.{fallocate_*,full_scrub_*,files*,kbytes*,mntdev}",
            "osd-*.*.{nonrotational,oi_scrub,pdo,stats,read*,write*}"
        ])
        self.get_params("quota", ["osd-*.*.quota_slave.{info,limit_*,acct_*}"])
        self.get_params("mgs", ["mgs.MGS.ir_timeout", "mgs.MGS.live.*"])

        # mb_groups can be VERY large, and provide minimal debug usefulness
        self.add_forbidden_path("*/mb_groups")
        self.add_copy_spec([
            "/sys/fs/ldiskfs",
            "/proc/fs/ldiskfs",
        ])

        # Grab emergency ring buffer dumps and other largish info
        if self.get_option("all_logs"):
            self.add_copy_spec("/tmp/lustre-log.*")
            self.get_params("job-stats", ["*.*.job_stats"])
            self.get_params("peers", ["peers"])

# vim: set et ts=4 sw=4 :



# Copyright (C) 2021 Mirntis, Inc., Oleksii Molchanov <omolchanov@mirantis.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class OpenContrail(Plugin, IndependentPlugin):
    short_desc = "OpenContrail SDN"
    plugin_name = 'opencontrail'
    profiles = ("network",)
    packages = ('opencontrail',)
    containers = (
        'opencontrail.*',
        'vrouter.*',
    )

    def setup(self):
        # assuming the container names will start with "opencontrail"
        in_container = self.container_exists('opencontrail.*')
        if in_container:
            cnames = self.get_containers(get_all=True)
            cnames = [c[1] for c in cnames if 'opencontrail' in c[1]]
            for cntr in cnames:
                self.add_cmd_output('contrail-status', container=cntr)
        else:
            self.add_cmd_output("contrail-status")

        self.add_cmd_output("vif --list")

        self.add_copy_spec([
            "/etc/contrail/*",
            "/var/log/contrail/*",
        ])

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from datetime import datetime
from sos.report.plugins import Plugin, RedHatPlugin


class Tomcat(Plugin, RedHatPlugin):

    short_desc = 'Apache Tomcat Server'
    plugin_name = 'tomcat'
    profiles = ('webserver', 'java', 'services', 'sysmgmt')

    packages = ('tomcat', 'tomcat6', 'tomcat7', 'tomcat8',
                'pki-servlet-engine')

    def setup(self):
        self.add_copy_spec([
            "/etc/tomcat",
            "/etc/tomcat6",
            "/etc/tomcat7",
            "/etc/tomcat8"
        ])
        self.add_forbidden_path("/etc/tomcat*/keystore")

        if not self.get_option("all_logs"):
            log_glob = "/var/log/tomcat*/catalina.out"
            self.add_copy_spec(log_glob)

            # get today's date in iso format so that days/months below 10
            # prepend 0
            today = datetime.date(datetime.now()).isoformat()
            log_glob = f"/var/log/tomcat*/catalina.{today}.log"
            self.add_copy_spec(log_glob)
        else:
            self.add_copy_spec("/var/log/tomcat*/*")

        self.add_file_tags({
            "/etc/tomcat.*/web.xml": "tomcat_web_xml",
            "/var/log/tomcat.*/catalina.out": "catalina_out",
            "/var/log/tomcat.*/catalina.*.log": "catalina_server_log"
        })

    def postproc(self):
        server_password_attr = ['keyPass', 'keystorePass',
                                'truststorePass', 'SSLPassword']
        self.do_path_regex_sub(
            r"\/etc\/tomcat.*\/server.xml",
            fr"({'|'.join(server_password_attr)})=(\S*)",
            r'\1="********"'
        )
        self.do_path_regex_sub(
            r"\/etc\/tomcat.*\/tomcat-users.xml",
            r"(password)=(\S*)",
            r'\1="********"'
        )

# vim: set et ts=4 sw=4 :



# Copyright (C) 2014 Red Hat, Inc., Sandro Bonazzola <sbonazzo@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class OvirtHostedEngine(Plugin, RedHatPlugin):

    short_desc = 'oVirt Hosted Engine'

    packages = (
        'ovirt-hosted-engine-setup',
        'ovirt-hosted-engine-ha',
    )

    plugin_name = 'ovirt_hosted_engine'
    profiles = ('virt',)

    def setup(self):
        # Add configuration files
        # Collecting the whole directory since it may contain branding
        # configuration files or third party plugins configuration files
        self.add_copy_spec(['/etc/ovirt-hosted-engine-setup.env.d/'])

        self.add_copy_spec([
            '/etc/ovirt-hosted-engine/answers.conf',
            '/etc/ovirt-hosted-engine/hosted-engine.conf',
            '/etc/ovirt-hosted-engine/vm.conf',
            '/etc/ovirt-hosted-engine-ha/agent.conf',
            '/etc/ovirt-hosted-engine-ha/agent-log.conf',
            '/etc/ovirt-hosted-engine-ha/broker.conf',
            '/etc/ovirt-hosted-engine-ha/broker-log.conf',
            '/etc/ovirt-hosted-engine-ha/notifications/state_transition.txt',
            '/run/ovirt-hosted-engine-ha/vm.conf',
            '/var/run/ovirt-hosted-engine-ha/vm.conf',
            '/var/lib/ovirt-hosted-engine-ha/broker.conf',
        ])

        self.add_copy_spec([
            '/var/log/ovirt-hosted-engine-setup',
            '/var/log/ovirt-hosted-engine-ha/agent.log*',
            '/var/log/ovirt-hosted-engine-ha/broker.log*',
        ])

        # Add gluster deployment and cleanup log
        self.add_copy_spec([
            '/var/log/cockpit/ovirt-dashboard'
        ])

        # Add run-time status
        self.add_cmd_output([
            'hosted-engine --vm-status',
            'hosted-engine --check-liveliness',
        ])


# vim: expandtab tabstop=4 shiftwidth=4



# Copyright (C) 2018 Masco Kaliyamoorthy <mkaliyam@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import os
from sos.report.plugins import Plugin, RedHatPlugin, PluginOpt


class Skydive(Plugin, RedHatPlugin):

    short_desc = 'Skydive network topology and protocol analyzer'

    plugin_name = "skydive"
    profiles = ('network', )
    files = (
        '/usr/bin/skydive',
        '/etc/skydive/skydive.yml'
    )

    password_warn_text = " (password visible in process listings)"

    option_list = [
        PluginOpt('username', default='', val_type=str,
                  desc='skydive username'),
        PluginOpt('password', default='', val_type=str,
                  desc='skydive password' + password_warn_text),
        PluginOpt('analyzer', default='', val_type=str,
                  desc='skydive analyzer address')
    ]

    def setup(self):
        self.add_copy_spec("/etc/skydive/skydive.yml")
        self.add_copy_spec("/var/log/skydive.log")

        username = (self.get_option("username") or
                    os.getenv("SKYDIVE_USERNAME", "") or
                    os.getenv("OS_USERNAME", ""))
        password = (self.get_option("password") or
                    os.getenv("SKYDIVE_PASSWORD", "") or
                    os.getenv("OS_PASSWORD", ""))
        analyzer = (self.get_option("analyzer") or
                    os.getenv("SKYDIVE_ANALYZER", "localhost:8082"))

        if not all([username, password, analyzer]):
            self.soslog.warning("Some or all of the skydive params are not "
                                "set properly. Skydive status command may "
                                " not work as expected.")

        # Setting all the params in environment variable for
        # skydive client access.
        os.environ["SKYDIVE_USERNAME"] = username
        os.environ["SKYDIVE_PASSWORD"] = password
        os.environ["SKYDIVE_ANALYZER"] = analyzer
        status_cmd = "skydive client status"
        self.add_cmd_output(status_cmd)

# vim: set et ts=4 sw=4 :



# Copyright (C) 2016 Red Hat, Inc., Tomas Tomecek <ttomecek@redhat>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.
import os

from sos.report.plugins import Plugin, IndependentPlugin, PluginOpt


class Npm(Plugin, IndependentPlugin):

    short_desc = 'Information from available npm modules'
    plugin_name = 'npm'
    profiles = ('system',)
    option_list = [
        PluginOpt('project-path', default='', val_type=str,
                  desc='Collect npm modules of project at this path')
    ]

    # in Fedora, Debian, Ubuntu and Suse the package is called npm
    packages = ('npm',)

    def _get_npm_output(self, cmd, filename, working_directory=None):
        # stderr output is already part of the json, key "problems"
        self.add_cmd_output(
            cmd,
            suggest_filename=filename,
            stderr=False,
            runat=working_directory
        )

    def setup(self):
        if self.get_option("project-path"):
            project_path = os.path.abspath(os.path.expanduser(
                self.get_option("project-path")))
            self._get_npm_output("npm ls --json", "npm_ls_project",
                                 working_directory=project_path)
            self._get_npm_output("npm config list -l",
                                 "npm_config_list_project",
                                 working_directory=project_path)

        self._get_npm_output("npm ls -g --json", "npm_ls_global")
        self._get_npm_output("npm config list -l", "npm_config_list_global")


class NpmViaNodeJS(Npm):
    """
    some distribution methods don't provide 'npm' via npm package
    """

    # upstream doesn't have an npm package, it's just nodejs
    # also in Fedora 24+ it is just nodejs, no npm package
    packages = ('nodejs', )

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.


from sos_tests import StageTwoReportTest


class OptionsConfigPresetCmdlineTest(StageTwoReportTest):
    """Ensure that we handle options specified in sos.conf and
    merge them with cmdline and preset properly

    :avocado: tags=stagetwo
    """

    files = [('options_tests_sos.conf', '/etc/sos/sos.conf')]
    sos_cmd = '--preset minimal --journal-size 20 --cmd-timeout 60'

    def test_case_id_from_config(self):
        self.assertTrue('8675309' in self.archive)

    def test_plugins_only_from_config(self):
        self.assertOnlyPluginsIncluded(['host', 'kernel'])

    def test_plugopts_logged_from_config(self):
        self.assertSosLogContains(
            r"Set kernel plugin option to \(name=with-timer, "
            r"desc='gather /proc/timer\* statistics', value=True, "
            r"default=False\)"
        )
        self.assertSosLogContains(
            r"Set kernel plugin option to \(name=trace, "
            "desc='gather /sys/kernel/debug/tracing/trace file', "
            r"value=True, default=False\)"
        )

    def test_disabled_plugopts_not_loaded(self):
        self.assertSosLogNotContains("Set networking plugin option to")

    def test_plugopts_actually_set(self):
        self.assertFileCollected('sys/kernel/debug/tracing/trace')

    def test_effective_options_logged_correctly(self):
        for option in [
            "--case-id 8675309",
            "--only-plugins host,kernel",
            "--plugopts kernel.with-timer=on,kernel.trace=yes",
            "--preset minimal",
            "--cmd-timeout 60",      # cmdline beats config and preset
            "--journal-size 20",     # cmdline beats preset
            "--log-size 10",         # preset setting is honored
            "--plugin-timeout 30",   # preset beats config file
        ]:
            self.assertSosLogContains(f"effective options now: .* {option}")


class PlugOptsConfigPresetCmdlineTest(StageTwoReportTest):
    """Ensure that plugin options specified in sos.conf or preset or cmdline
    are handled and merged properly.

    :avocado: tags=stagetwo
    """

    files = [
        ('options_tests_sos.conf', '/etc/sos/sos.conf'),
        ('options_tests_preset.json', '/etc/sos/presets.d/plugopts_preset')
    ]
    sos_cmd = '--preset plugopts_preset --container-runtime=none ' \
              '-k crio.timeout=10,networking.timeout=20 -o crio,networking'
    redhat_only = True

    def test_effective_plugopts_logged_correctly(self):
        for option in [
            "--only-plugins crio,networking",
            "--preset plugopts_preset",
            "networking.timeout=20",                # cmd beats config&preset
            "crio.timeout=10",                      # cmdline beats preset
            "networking.ethtool-namespaces=False",  # preset setting is honored
            "networking.namespaces=100",            # preset beats config file
        ]:
            self.assertSosLogContains(f"effective options now: .*{option}")



# Copyright (C) 2023 Canonical Ltd., Arif Ali <arif.ali@canonical.com>
#
# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, UbuntuPlugin


class Vault(Plugin, UbuntuPlugin):
    """The Vault plugin collects the current status of the vault
    snap on a Ubuntu machine.

    It will collect logs from journal, vault status and configuration
    """

    short_desc = 'Manage access to secrets and protect sensitive data'

    plugin_name = 'vault'
    profiles = ('sysmgmt', 'security')
    services = ('vault',)
    package = ('vault',)

    def setup(self):

        vault_cfg = "/var/snap/vault/common/vault.hcl"

        self.add_copy_spec(vault_cfg)

        try:
            with open(vault_cfg, 'r', encoding='UTF-8') as cfile:
                for line in cfile.read().splitlines():
                    if not line:
                        continue
                    words = line.split('=')
                    if words[0].strip() == 'api_addr':
                        api_addr = words[1].strip('\" ')
                        self.add_cmd_output("vault status",
                                            env={'VAULT_ADDR': api_addr})
        except IOError as error:
            self._log_error(f'Could not open conf file {vault_cfg}: {error}')

    def postproc(self):
        self.do_file_sub(
            "/var/snap/vault/common/vault.hcl",
            r"(password\s?=\s?).*",
            r"\1******"
        )

# vim: set et ts=4 sw=4 :



# Copyright 2020 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

# pylint: disable=no-member

import re


class SoSCleanerParser():
    """Parsers are used to build objects that will take a line as input,
    parse it for a particular pattern (E.G. IP addresses) and then make any
    necessary subtitutions by referencing the SoSMap() associated with the
    parser.

    Ideally a new parser subclass will only need to set the class level attrs
    in order to be fully functional.

    :param conf_file: The configuration file to read from
    :type conf_file: ``str``

    :cvar name:     The parser name, used in logging errors
    :vartype name: ``str``

    :cvar regex_patterns:   A list of regex patterns to iterate over for every
                            line processed
    :vartype regex_patterns: ``list``

    :cvar mapping: Used by the parser to store and obfuscate matches
    :vartype mapping: ``SoSMap()``


    :cvar map_file_key: The key in the ``map_file`` to read when loading
                        previous obfuscation matches
    :vartype map_file_key: ``str``
    """

    name = 'Undefined Parser'
    regex_patterns = []
    skip_line_patterns = []
    parser_skip_files = []  # list of skip files relevant to a parser
    skip_cleaning_files = []   # list of global skip files from cmdline args
    map_file_key = 'unset'
    compile_regexes = True

    def __init__(self, config={}, skip_cleaning_files=[]):
        if self.map_file_key in config:
            self.mapping.conf_update(config[self.map_file_key])
        self.skip_cleaning_files = skip_cleaning_files
        self._generate_skip_regexes()

    def load_map_entries(self):
        self.mapping.load_entries()

    def _generate_skip_regexes(self):
        """Generate the regexes for the parser's configured parser_skip_files
        or global skip_cleaning_files, so that we don't regenerate them on
        every file being examined for if the parser should skip a given file.
        """
        self.skip_patterns = []
        for p in self.parser_skip_files + self.skip_cleaning_files:
            self.skip_patterns.append(re.compile(p))

    def generate_item_regexes(self):
        """Generate regexes for items the parser will be searching for
        repeatedly without needing to generate them for every file and/or line
        we process

        Not used by all parsers.
        """
        if not self.compile_regexes:
            return
        for obitem in self.mapping.dataset:
            self.mapping.add_regex_item(obitem)

    def parse_line(self, line):
        """This will be called for every line in every file we process, so that
        every parser has a chance to scrub everything.

        This will first try to identify needed obfuscations for items we have
        already encountered (if the parser uses compiled regexes that is) and
        make those substitutions early on. After which, we will then parse the
        line again looking for new matches.
        """
        count = 0
        for skip_pattern in self.skip_line_patterns:
            if re.match(skip_pattern, line, re.I):
                return line, count
        if self.compile_regexes:
            line, _rcount = self._parse_line_with_compiled_regexes(line)
            count += _rcount
        line, _count = self._parse_line(line)
        count += _count
        return line, count

    def _parse_line_with_compiled_regexes(self, line):
        """Check the provided line against known items we have encountered
        before and have pre-generated regex Pattern() objects for.

        :param line:    The line to parse for possible matches for obfuscation
        :type line:     ``str``

        :returns:   The obfuscated line and the number of changes made
        :rtype:     ``str``, ``int``
        """
        count = 0
        for item, reg in self.mapping.compiled_regexes:
            if reg.search(line):
                line, _count = reg.subn(self.mapping.get(item.lower()), line)
                count += _count
        return line, count

    def _parse_line(self, line):
        """Check the provided line against the parser regex patterns to try
        and discover _new_ items to obfuscate

        :param line: The line to parse for possible matches for obfuscation
        :type line: ``str``

        :returns: The obfsucated line, and the number of changes made
        :rtype: ``tuple``, ``(str, int))``
        """
        count = 0
        for pattern in self.regex_patterns:
            matches = [m[0] for m in re.findall(pattern, line, re.I)]
            if matches:
                matches.sort(reverse=True, key=len)
                count += len(matches)
                for match in matches:
                    match = match.strip()
                    if match in self.mapping.dataset.values():
                        continue
                    new_match = self.mapping.get(match)
                    if new_match != match:
                        line = line.replace(match, new_match)
        return line, count

    def parse_string_for_keys(self, string_data):
        """Parse a given string for instances of any obfuscated items, without
        applying the normal regex comparisons first. This is mainly used to
        obfuscate filenames that have, for example, hostnames in them.

        Rather than try to regex match the string_data, just use the builtin
        checks for substrings matching known obfuscated keys

        :param string_data: The line to be parsed
        :type string_data: ``str``

        :returns: The obfuscated line
        :rtype: ``str``
        """
        if self.compile_regexes:
            for item, reg in self.mapping.compiled_regexes:
                if reg.search(string_data):
                    string_data = reg.sub(self.mapping.get(item.lower()),
                                          string_data)
        else:
            for k, ob in sorted(self.mapping.dataset.items(), reverse=True,
                                key=lambda x: len(x[0])):
                if k in self.mapping.skip_keys:
                    continue
                if k in string_data:
                    string_data = string_data.replace(k, ob)
        return string_data

    def get_map_contents(self):
        """Get the contents of the mapping used by the parser

        :returns: All matches and their obfuscate counterparts
        :rtype: ``dict``
        """
        return self.mapping.dataset



# Copyright Red Hat 2021, Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import os
import shutil

from sos.collector.transports import RemoteTransport


class LocalTransport(RemoteTransport):
    """
    A 'transport' to represent a local node. No remote connection is actually
    made, and all commands set to be run by this transport are executed locally
    without any wrappers.
    """

    name = 'local_node'

    def _connect(self, password):
        return True

    def _disconnect(self):
        return True

    @property
    def connected(self):
        return True

    def _retrieve_file(self, fname, dest):
        self.log_debug(f"Moving {fname} to {dest}")
        shutil.copy(fname, dest)
        return True

    def _format_cmd_for_exec(self, cmd):
        return cmd

    def _copy_file_to_remote(self, fname, dest):
        return True

    def _read_file(self, fname):
        if os.path.exists(fname):
            with open(fname, 'r', encoding='utf-8') as rfile:
                return rfile.read()
        self.log_debug(f"No such file: {fname}")
        return ''

# vim: set et ts=4 sw=4 :



# Copyright (C) 2017 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class Rear(Plugin, RedHatPlugin):

    short_desc = 'Relax and Recover'
    plugin_name = "rear"
    packages = ('rear',)

    def setup(self):
        # don't collect recovery ISOs or tar archives
        self.add_forbidden_path([
            '/var/lib/rear/output/*'
        ])

        self.add_copy_spec([
            '/etc/rear/*conf',
            '/etc/rear/mappings/*',
            '/var/lib/rear/layout/*',
            '/var/lib/rear/recovery/*',
            '/var/log/rear/*log*',
            '/var/log/rear/recover/*log*',
            '/var/log/rear/recover/restore/*log*'
        ])

        self.add_cmd_output([
            'rear -V',
            'rear dump'
        ])

    def postproc(self):
        self.do_path_regex_sub(
            '/etc/rear/*',
            r'(SSH_ROOT_PASSWORD)=(.*)',
            r'\1=********'
        )

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.


from sos_tests import StageOneReportTest


class CommandPriorityTest(StageOneReportTest):
    """Ensure that the priority parameter for command execution is functioning
    as expected

    :avocado: tags=stageone
    """

    sos_cmd = '-o logs,process'

    def test_logs_full_journal_correct_priority(self):
        cmds = self.get_plugin_manifest('logs')['commands']
        fullj = cmds[-1]
        self.assertEqual(fullj['priority'], 100)

    def test_logs_full_journal_run_last(self):
        cmds = self.get_plugin_manifest('logs')['commands']
        cmds.sort(key=lambda x: x['start_time'])
        # journal_full should be the last command executed
        self.assertTrue('journal_full' in cmds[-1]['tags'])

    def test_process_correct_priorities(self):
        cmds = self.get_plugin_manifest('process')['commands']
        # ensure root symlinked ps ran first
        self.assertTrue(
            cmds[0]['priority'] == 1 and 'ps_aux' in cmds[0]['tags']
        )

        # get lsof and iotop command entries
        _lsof = None
        _iotop = None
        for cmd in cmds:
            if cmd['command'] == 'lsof':
                _lsof = cmd
            elif cmd['command'] == 'iotop':
                _iotop = cmd

        self.assertTrue(_lsof and _iotop, "lsof or iotop output missing")

        self.assertEqual(_lsof['priority'], 50)
        self.assertEqual(_iotop['priority'], 100)
        self.assertTrue(_lsof['start_time'] < _iotop['start_time'])



# Copyright 2023 Red Hat, Inc. Evgeny Slutsky <eslutsky@redhat.com>
# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class Greenboot(Plugin, RedHatPlugin):
    """The greenboot plugin collects systemd service logs and configuration.
    """

    short_desc = 'Greenboot'
    plugin_name = 'greenboot'
    services = (plugin_name, 'greenboot-healthcheck',
                'greenboot-task-runner', 'redboot-task-runner',)
    profiles = ('system',)

    def setup(self):
        self.add_copy_spec([
            "/etc/greenboot/greenboot.conf",
        ])



# Copyright (C) 2023 Canonical Ltd., Nikhil Kshirsagar <nkshirsagar@ubuntu.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import os

from sos.report.plugins import Plugin, RedHatPlugin, UbuntuPlugin


class CephMGR(Plugin, RedHatPlugin, UbuntuPlugin):
    """
    This plugin is for capturing information from Ceph mgr nodes. While the
    majority of this plugin should be version-agnostic, several collections are
    dependent upon the version of Ceph installed. Versions that correlate to
    RHCS 4 or RHCS 5 are explicitly handled for differences such as those
    pertaining to log locations on the host filesystem.

    Note that while this plugin will activate based on the presence of Ceph
    containers, commands are run directly on the host as those containers are
    often not configured to successfully run the `ceph` commands collected by
    this plugin. These commands are majorily `ceph daemon` commands that will
    reference discovered admin sockets under /var/run/ceph.

    Users may expect to see several collections twice - once in standard output
    from the `ceph` command, and again in JSON format. The latter of which will
    be placed in the `json_output/` subdirectory within this plugin's directory
    in the report archive. These JSON formatted collections are intended to
    aid in automated analysis.
    """

    short_desc = 'CEPH mgr'

    plugin_name = 'ceph_mgr'
    profiles = ('storage', 'virt', 'container', 'ceph')
    files = ('/var/lib/ceph/mgr/*', '/var/lib/ceph/*/mgr*',
             '/var/snap/microceph/common/data/mgr/*')
    containers = ('ceph-(.*-)?mgr.*',)

    def setup(self):
        all_logs = self.get_option("all_logs")
        microceph_pkg = self.policy.package_manager.pkg_by_name('microceph')

        ceph_mgr_cmds = ([
            "balancer eval",
            "balancer status",  # For pre-Squid releases
            "balancer status detail",
            "healthcheck history ls",
            "log last cephadm",
            "mgr dump",
            "mgr metadata",
            "mgr module ls",
            "mgr services",
            "mgr stat",
            "mgr versions"
        ])

        # if orchestrator is configured
        orch_configured = self.exec_cmd('ceph orch status')
        if orch_configured['status'] == 0:
            ceph_mgr_cmds += ([
                "orch host ls",
                "orch device ls",
                "orch ls",
                "orch ls --export",
                "orch ps",
                "orch status --detail",
                "orch upgrade status"
            ])

        cmds = [
            "config diff",
            "config show",
            "counter dump",
            "counter schema",
            "dump_cache",
            "dump_mempools",
            "dump_osd_network",
            "mds_requests",
            "mds_sessions",
            "objecter_requests",
            "perf dump",
            "perf histogram dump",
            "perf histogram schema",
            "perf schema",
            "status",
            "version",
        ]

        directory = ''
        if not microceph_pkg:
            directory = '/var/run/ceph'
            self.add_file_tags({
                '/var/log/ceph/(.*/)?ceph-mgr.*.log': 'ceph_mgr_log',
            })

            self.add_forbidden_path([
                "/etc/ceph/*keyring*",
                "/var/lib/ceph/**/*keyring*",
                "/var/lib/ceph/**/osd*",
                "/var/lib/ceph/**/mon*",
                # Excludes temporary ceph-osd mount location like
                # /var/lib/ceph/tmp/mnt.XXXX from sos collection.
                "/var/lib/ceph/**/tmp/*mnt*",
                "/etc/ceph/*bindpass*",
            ])

            if not all_logs:
                self.add_copy_spec([
                    "/var/log/ceph/**/ceph-mgr*.log",
                ])
            else:
                self.add_copy_spec([
                    "/var/log/ceph/**/ceph-mgr*.log*",
                ])

            self.add_copy_spec([
                "/var/lib/ceph/**/mgr*",
                "/var/lib/ceph/**/bootstrap-mgr/",
                "/run/ceph/**/ceph-mgr*",
            ])

        else:
            directory = '/var/snap/microceph/current/run'
            self.add_file_tags({
                '/var/snap/microceph/common/logs/ceph-mgr.*.log':
                'ceph_mgr_log',
            })

            self.add_forbidden_path([
                "/var/snap/microceph/common/**/*keyring*",
            ])

            if not all_logs:
                self.add_copy_spec([
                    "/var/snap/microceph/common/logs/ceph-mgr*.log",
                ])
            else:
                self.add_copy_spec([
                    "/var/snap/microceph/common/logs/ceph-mgr*.log*",
                ])

        self.add_cmd_output(
            [f"ceph {cmd}" for cmd in ceph_mgr_cmds])

        # get ceph_cmds again as json for easier automation parsing
        self.add_cmd_output(
            [f"ceph {cmd} --format json-pretty" for cmd in ceph_mgr_cmds],
            subdir="json_output",
        )

        self.add_cmd_output([
            f"ceph daemon {m} {cmd}" for m in self.get_socks(directory)
            for cmd in cmds]
        )

    def get_socks(self, directory):
        """
        Find any available admin sockets under /var/run/ceph (or subdirs for
        later versions of Ceph) which can be used for ceph daemon commands
        """
        ceph_sockets = []
        for rdir, _, files in os.walk(directory):
            for file in files:
                if file.startswith('ceph-mgr') and file.endswith('.asok'):
                    ceph_sockets.append(self.path_join(rdir, file))
        return ceph_sockets

# vim: set et ts=4 sw=4 :



# Copyright (C) 2014 Red Hat, Inc., Bryn M. Reeves <bmr@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class DistUpgrade(Plugin):

    short_desc = 'Distribution upgrade data'

    plugin_name = "distupgrade"
    profiles = ('system', 'sysmgmt')


class RedHatDistUpgrade(DistUpgrade, RedHatPlugin):

    packages = (
        'preupgrade-assistant',
        'preupgrade-assistant-ui',
        'preupgrade-assistant-el6toel7',
        'redhat-upgrade-tool'
    )

    files = (
        "/var/log/upgrade.log",
        "/var/log/redhat_update_tool.log",
        "/root/preupgrade/all-xccdf*",
        "/root/preupgrade/kickstart"
    )

    def postproc(self):
        self.do_file_sub(
            "/root/preupgrade/kickstart/anaconda-ks.cfg",
            r"(useradd --password) (.*)",
            r"\1 ********"
        )

        self.do_file_sub(
            "/root/preupgrade/kickstart/anaconda-ks.cfg",
            r"(\s*rootpw\s*).*",
            r"\1********"
        )

        self.do_file_sub(
            "/root/preupgrade/kickstart/untrackeduser",
            r"\/home\/.*",
            r"/home/******** path redacted ********"
        )

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import (Plugin, RedHatPlugin, DebianPlugin,
                                UbuntuPlugin, PluginOpt)


class Psacct(Plugin):

    short_desc = 'Process accounting information'
    plugin_name = "psacct"
    profiles = ('system',)

    option_list = [
        PluginOpt('all', default=False, desc='collect all accounting files')
    ]

    packages = ("psacct", )


class RedHatPsacct(Psacct, RedHatPlugin):

    packages = ("psacct", )

    def setup(self):
        super().setup()
        self.add_copy_spec("/var/account/pacct")
        if self.get_option("all"):
            self.add_copy_spec("/var/account/pacct*.gz")


class DebianPsacct(Psacct, DebianPlugin, UbuntuPlugin):

    packages = ("acct", )

    def setup(self):
        super().setup()
        self.add_copy_spec(["/var/log/account/pacct", "/etc/default/acct"])
        if self.get_option("all"):
            self.add_copy_spec("/var/log/account/pacct*.gz")

# vim: set et ts=4 sw=4 :



# Copyright (C) 2024 Jake Hunsaker <jacob.r.hunsaker@gmail.com>
# Copyright (C) 2019 Alexander Petrovskiy <alexpe@mellanox.com>
#
# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Bird(Plugin, IndependentPlugin):
    """BIRD is an Internet Routing Daemon used in many *nix and nix-like
    distributions. This plugin will capture the configuration files for a local
    bird installation, as well as runtime information and metrics.
    """

    plugin_name = 'bird'
    profiles = ('network', )
    packages = ('bird', )
    services = ('bird', )

    def setup(self):

        try:
            with open('/etc/bird.conf', 'r', encoding='utf-8') as bfile:
                for line in bfile:
                    if line.startswith('log'):
                        # non-file values will be dropped by add_copy_spec()
                        self.add_copy_spec(line.split()[1].strip('"'))
        except Exception as err:
            self._log_debug(f"Unable to parse bird.conf: {err}")

        self.add_copy_spec([
            "/etc/bird/*",
            "/etc/bird.conf"
        ])

        self.add_cmd_output([
            "birdc show status",
            "birdc show memory",
            "birdc show protocols all",
            "birdc show interfaces",
            "birdc show route all",
            "birdc show symbols",
            "birdc show bfd sessions",
            "birdc show babel interfaces",
            "birdc show babel neighbors",
            "birdc show babel entries",
            "birdc show babel routes",
            "birdc show ospf",
            "birdc show ospf neighbors",
            "birdc show ospf interface",
            "birdc show ospf topology",
            "birdc show ospf state all",
            "birdc show ospf lsadb",
            "birdc show rip interfaces",
            "birdc show rip neighbors",
            "birdc show static"
        ])

    def postproc(self):
        self.do_path_regex_sub('/etc/bird(.*)?.conf',
                               r"((.*password)\s\"(.*)\"(.*))",
                               r"\2 *******\4")

# vim: set et ts=4 sw=4 :



# Copyright (C) 2020 Red Hat, Inc., Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.options import SoSOptions
from sos.presets import PresetDefaults


RHEL_RELEASE_STR = "Red Hat Enterprise Linux"

_opts_verify = SoSOptions(verify=True)
_cb_profiles = ['boot', 'storage', 'system']
_cb_plugopts = ['boot.all-images=on', 'rpm.rpmva=on', 'rpm.rpmdb=on']


RHV = "rhv"
RHV_DESC = "Red Hat Virtualization"

RHEL = "rhel"
RHEL_DESC = RHEL_RELEASE_STR

RHOSP = "rhosp"
RHOSP_DESC = "Red Hat OpenStack Platform"
RHOSP_OPTS = SoSOptions(plugopts=[
                             'process.lsof=off',
                             'networking.ethtool-namespaces=False',
                             'networking.namespaces=200'])

RHOCP = "ocp"
RHOCP_DESC = "OpenShift Container Platform by Red Hat"
RHOCP_OPTS = SoSOptions(
    skip_plugins=['cgroups'], container_runtime='crio', no_report=True,
    log_size=100, enable_plugins=['openshift', 'openshift_ovn'],
    plugopts=[
        'crio.timeout=600',
        'crio.all=on',
        'crio.logs=on',
        'podman.all=on',
        'podman.logs=on',
        'networking.timeout=600',
        'networking.ethtool-namespaces=False',
        'networking.namespaces=200'
    ])

RH_CFME = "cfme"
RH_CFME_DESC = "Red Hat CloudForms"

RH_SATELLITE = "satellite"
RH_SATELLITE_DESC = "Red Hat Satellite"
SAT_OPTS = SoSOptions(log_size=100, plugopts=['apache.log=on'])

AAPEDA = 'aap_eda'
AAPEDA_DESC = 'Ansible Automation Platform Event Driven Controller'
AAPEDA_OPTS = SoSOptions(
    enable_plugins=['containers_common'],
    plugopts=[
        'containers_common.rootlessusers=eda'
    ])
AAPEDA_NOTE = ('Collects \'eda\' user output for the containers_common plugin.'
               ' If you need more users, do not forget to add \'eda\' '
               'to your own list for the \'rootlessusers\' option.')

AAPCONTROLLER = 'aap_controller'
AAPCONTROLLER_DESC = 'Ansible Automation Platform Controller'
AAPCONTROLLER_OPTS = SoSOptions(
    enable_plugins=['containers_common'],
    plugopts=[
        'containers_common.rootlessusers=awx'
    ])
AAPCONTROLLER_NOTE = ('Collects \'awx\' user output for the containers_common'
                      'pluging. If you need more users, do not forget to add'
                      '\'awx\' to your own list for the \'rootlessusers\' '
                      'option.')

CB = "cantboot"
CB_DESC = "For use when normal system startup fails"
CB_OPTS = SoSOptions(
            verify=True, all_logs=True, profiles=_cb_profiles,
            plugopts=_cb_plugopts
          )
CB_NOTE = "Data collection will be limited to a boot-affecting scope"

NOTE_SIZE = "This preset may increase report size"
NOTE_TIME = "This preset may increase report run time"
NOTE_SIZE_TIME = "This preset may increase report size and run time"

RHEL_PRESETS = {
    AAPEDA: PresetDefaults(name=AAPEDA, desc=AAPEDA_DESC, opts=AAPEDA_OPTS,
                           note=AAPEDA_NOTE),
    AAPCONTROLLER: PresetDefaults(name=AAPCONTROLLER, desc=AAPCONTROLLER_DESC,
                                  opts=AAPCONTROLLER_OPTS,
                                  note=AAPCONTROLLER_NOTE),
    RHV: PresetDefaults(name=RHV, desc=RHV_DESC, note=NOTE_TIME,
                        opts=_opts_verify),
    RHEL: PresetDefaults(name=RHEL, desc=RHEL_DESC),
    RHOSP: PresetDefaults(name=RHOSP, desc=RHOSP_DESC, opts=RHOSP_OPTS),
    RHOCP: PresetDefaults(name=RHOCP, desc=RHOCP_DESC, note=NOTE_SIZE_TIME,
                          opts=RHOCP_OPTS),
    RH_CFME: PresetDefaults(name=RH_CFME, desc=RH_CFME_DESC, note=NOTE_TIME,
                            opts=_opts_verify),
    RH_SATELLITE: PresetDefaults(name=RH_SATELLITE, desc=RH_SATELLITE_DESC,
                                 note=NOTE_TIME, opts=SAT_OPTS),
    CB: PresetDefaults(name=CB, desc=CB_DESC, note=CB_NOTE, opts=CB_OPTS)
}


# vim: set et ts=4 sw=4 :



# Copyright (C) 2025 Canonical Ltd.,
#                    Bryan Fraschetti <bryan.fraschetti@canonical.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class PulseAudio(Plugin, IndependentPlugin):
    """The PulseAudio plugin collects information about the system's inputs
    sources, output sinks, detected sound cards, and pulse audio's
    configuration
    """

    short_desc = 'The sound server audio middleware'
    plugin_name = "pulseaudio"
    profiles = ('system', 'desktop', 'hardware')

    packages = ('pulseaudio-utils', 'pulseaudio')

    pactl_cmd = "pactl"
    pulseaudio_cmd = "pulseaudio"

    def setup(self):
        pactl_subcmds = [
            'list sinks',
            'list sources',
            'list cards',
            'info',
            'stat',
            '--version'
        ]

        pulseaudio_subcmds = [
            '--dump-conf',
            '--dump-modules',
            '--check'
        ]

        self.add_cmd_output([
            f"{self.pactl_cmd} {subcmd}" for subcmd in pactl_subcmds
        ])
        self.add_cmd_output([
            f"{self.pulseaudio_cmd} {subcmd}" for subcmd in pulseaudio_subcmds
        ])

        self.add_copy_spec("/etc/pulse/*")

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import json
import re

from sos.report.plugins import Plugin, IndependentPlugin, PluginOpt


class Process(Plugin, IndependentPlugin):

    short_desc = 'process information'

    plugin_name = 'process'
    profiles = ('system',)

    option_list = [
        PluginOpt('lsof', default=True, desc='collect info on all open files'),
        PluginOpt('lsof-threads', default=False,
                  desc='collect threads\' open file info if supported'),
        PluginOpt('smaps', default=False, desc='collect /proc/*/smaps files'),
        PluginOpt('samples', default=20, val_type=int,
                  desc='number of iotop samples to collect'),
        PluginOpt('numprocs', default=2048, val_type=int,
                  desc='number of process to collect /proc data of')
    ]

    def setup(self):
        ps_axo = "ps axo"
        # process group and thread options
        ps_group_opts = "pid,ppid,user,group,lwp,nlwp,start_time,comm,cgroup"
        ps_sched_opts = "flags,state,uid,pid,ppid,pgid,sid,cls,pri,psr,addr,"
        ps_sched_opts += "sz,wchan:20,lstart,tty,time,cmd"

        self.add_copy_spec([
            "/proc/sched_debug",
            "/proc/stat",
            "/sys/kernel/debug/sched/debug",
            "/sys/kernel/debug/sched/features",
            "/sys/kernel/sched_ext/",
        ])

        procs = [p for p in self.listdir("/proc") if re.match("[0-9]", p)]
        if self.get_option("numprocs"):
            procs = procs[:self.get_option("numprocs")]

        for proc in procs:
            self.add_copy_spec([
                f"/proc/{proc}/status",
                f"/proc/{proc}/cpuset",
                f"/proc/{proc}/oom_*",
                f"/proc/{proc}/stack",
                f"/proc/{proc}/limits",
            ])

        if self.get_option("smaps"):
            self.add_copy_spec("/proc/[0-9]*/smaps")

        self.add_cmd_output("ps auxwwwm", root_symlink="ps",
                            tags=['ps_aux', 'ps_auxww', 'ps_auxwww',
                                  'ps_auxwwwm', 'ps_auxcww'],
                            priority=1)

        self.add_cmd_output("pstree -lp", root_symlink="pstree")
        if self.get_option("lsof"):
            self.add_cmd_output("lsof +M -n -l -c ''", root_symlink="lsof",
                                timeout=15, priority=50, tags="lsof")

        if self.get_option("lsof-threads"):
            self.add_cmd_output("lsof +M -n -l", timeout=15, priority=50)

        self.add_cmd_output([
            "ps alxwww",
            "ps auxfwww",
            "ps -elfL"
        ], cmd_as_tag=True)

        self.add_cmd_output([
            f"{ps_axo} {ps_group_opts}",
            f"{ps_axo} {ps_sched_opts}",
        ])

        if self.get_option("samples"):
            self.add_cmd_output("iotop -b -o -d 0.5 -t -n "
                                f"{self.get_option('samples')}",
                                priority=100)

        self.add_cmd_output([
            "pidstat -p ALL -rudvwsRU --human -h",
            "pidstat -tl"
        ])

    def collect(self):
        with self.collection_file('pids_to_packages.json') as pfile:
            if not self.policy.package_manager.query_path_command:
                pfile.write('Package manager not configured for path queries')
                return
            _ps = self.exec_cmd('ps --no-headers aex')
            pidpkg = {}
            paths = {}
            if not _ps['status'] == 0:
                pfile.write(f"Unable to get process list: {_ps['output']}")
                return
            for proc in _ps['output'].splitlines():
                proc = proc.strip().split()
                pid = proc[0]
                path = proc[4]
                if not self.path_exists(path):
                    continue
                if path not in paths:
                    paths[path] = self.policy.package_manager.pkg_by_path(path)
                pidpkg[pid] = {'path': path, 'package': paths[path]}

            pfile.write(json.dumps(pidpkg, indent=4))

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from os.path import join, normpath
from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class Named(Plugin):

    short_desc = 'BIND named server'

    plugin_name = "named"
    profiles = ('system', 'services', 'network')
    named_conf = "/etc/named.conf"
    config_files = named_conf

    def setup(self):
        self.add_copy_spec([
            "/etc/default/bind",
            "/var/log/named*.log"
        ])
        for cfg in self.config_files:
            if self.path_exists(cfg):
                self.add_copy_spec([
                    cfg,
                    self.get_dns_dir(cfg)
                ])
                self.add_forbidden_path([
                    join(self.get_dns_dir(cfg), "chroot/dev"),
                    join(self.get_dns_dir(cfg), "chroot/proc")
                ])

    def get_dns_dir(self, config_file):
        """ grab directory path from named{conf,boot}
        """
        directory_list = self.do_regex_find_all(r"directory\s+\"(.*)\"",
                                                config_file)
        if directory_list:
            return normpath(directory_list[0])

        return ""

    def postproc(self):
        match = r"(\s*arg \"password )[^\"]*"
        subst = r"\1******"
        self.do_file_sub(self.named_conf, match, subst)


class RedHatNamed(Named, RedHatPlugin):

    named_conf = "/etc/named.conf"
    config_files = ("/etc/named.conf",
                    "/etc/named.boot")
    files = (named_conf, '/etc/sysconfig/named')
    packages = ('bind',)

    def setup(self):
        super().setup()
        self.add_copy_spec("/etc/named/")
        self.add_copy_spec("/etc/sysconfig/named")
        self.add_cmd_output("klist -ket /etc/named.keytab")
        self.add_forbidden_path("/etc/named.keytab")


class DebianNamed(Named, DebianPlugin, UbuntuPlugin):

    files = ('/etc/bind/named.conf',)
    packages = ('bind9',)
    named_conf = "/etc/bind/named.conf"
    config_files = (named_conf,
                    "/etc/bind/named.conf.options",
                    "/etc/bind/named.conf.local")

    def setup(self):
        super().setup()
        self.add_copy_spec("/etc/bind/")


# vim: set et ts=4 sw=4 :



# Copyright (c) 2024 Rudnei Bertol Jr <rudnei@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import glob
from sos.report.plugins import Plugin, RedHatPlugin


class AAPreceptorPlugin(Plugin, RedHatPlugin):
    short_desc = 'AAP receptor plugin'
    plugin_name = 'aap_receptor'
    profiles = ('sysmgmt', 'ansible')
    packages = ('receptor', 'receptorctl')
    services = ('receptor',)

    def setup(self):
        self.add_copy_spec([
            "/etc/receptor",
            "/tmp/receptor/*/status",
            "/var/lib/receptor",
        ])

        if self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/log/receptor"
            ])
        else:
            self.add_copy_spec([
                "/var/log/receptor/*.log"
            ])

        self.add_forbidden_path([
            "/etc/receptor/tls",
            "/etc/receptor/*key.pem"
        ])

        self.add_dir_listing([
            "/etc/receptor",
            "/var/run/receptor",
            "/var/run/awx-receptor"
        ])

        for s in glob.glob('/var/run/*receptor/*.sock'):
            self.add_cmd_output(f"receptorctl --socket {s} status",
                                suggest_filename="receptorctl_status")
            self.add_cmd_output(f"receptorctl --socket {s} status --json",
                                suggest_filename="receptorctl_status.json")
            self.add_cmd_output(f"receptorctl --socket {s} work list",
                                suggest_filename="receptorctl_work_list.json")
            break

# vim: set et ts=4 sw=4 :



# Copyright (C) 2007 Red Hat, Inc., Justin Payne <jpayne@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class S390(Plugin, IndependentPlugin):

    short_desc = 'IBM S/390'

    plugin_name = 's390'
    profiles = ('system', 'hardware')
    architectures = ('s390.*',)

    def setup(self):
        self.add_copy_spec([
            "/proc/cio_ignore",
            "/proc/crypto",
            "/proc/dasd/devices",
            "/proc/dasd/statistics",
            "/etc/dasd.conf",
            "/proc/qeth",
            "/proc/qeth_perf",
            "/proc/qeth_ipa_takeover",
            "/proc/sys/appldata/*",
            "/proc/sys/kernel/hz_timer",
            "/proc/sysinfo",
            "/sys/bus/ccwgroup/drivers/qeth/0.*/*",
            "/sys/bus/ccw/drivers/zfcp/0.*/*",
            "/sys/bus/ccw/drivers/zfcp/0.*/0x*/*",
            "/sys/bus/ccw/drivers/zfcp/0.*/0x*/0x*/*",
            "/sys/kernel/debug/s390dbf",
            "/etc/zipl.conf",
            "/etc/zfcp.conf",
            "/etc/sysconfig/dumpconf",
            "/etc/src_vipa.conf",
            "/etc/ccwgroup.conf",
            "/etc/chandev.conf",
            "/var/log/IBMtape.trace",
            "/var/log/IBMtape.errorlog",
            "/var/log/lin_tape.trace",
            "/var/log/lin_tape.errorlog",
        ])

        # skip flush as it is useless for sos collection
        self.add_forbidden_path("/sys/kernel/debug/s390dbf/*/flush")

        self.add_cmd_output([
            "lscss",
            "lsdasd",
            "lsshut",
            "lstape",
            "qethconf list_all",
            "lsqeth",
            "lszfcp",
            "lszfcp -D",
            "lszfcp -V",
            "lszcrypt -VV",
            "icainfo",
            "icastats",
            "smc_dbg"
        ])

        ret = self.exec_cmd("ls /dev/dasd?")
        dasd_dev = ret['output']
        for dev in dasd_dev.split('\n'):
            self.add_cmd_output([
                f"dasdview -x -i -j -l -f {dev}",
                f"fdasd -p {dev}",
            ])


# vim: set et ts=4 sw=4 :



# Copyright (C) 2009 Red Hat, Inc., Joey Boggs <jboggs@redhat.com>
# Copyright (C) 2012 Rackspace US, Inc.,
#                    Justin Shepherd <jshepher@rackspace.com>
# Copyright (C) 2013 Red Hat, Inc., Flavio Percoco <fpercoco@redhat.com>
# Copyright (C) 2013 Red Hat, Inc., Jeremy Agee <jagee@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import os
from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class OpenStackCinder(Plugin):

    short_desc = 'OpenStack cinder'
    plugin_name = "openstack_cinder"
    profiles = ('openstack', 'openstack_controller')
    containers = ('.*cinder_api',)

    var_puppet_gen = "/var/lib/config-data/puppet-generated/cinder"
    apachepkg = None

    def setup(self):
        self.add_forbidden_path('/etc/cinder/volumes')
        cinder_config = ""
        cinder_config_opt = "--config-dir %s/etc/cinder/"

        # check if either standalone (cinder-api) or httpd wsgi (cinder_wsgi)
        # is up and running
        cinder_process = ["cinder_wsgi", "cinder-wsgi", "cinder-api"]
        in_ps = False
        for process in cinder_process:
            in_ps = self.check_process_by_name(process)
            if in_ps:
                break

        in_container = self.container_exists('.*cinder_api')
        if in_container:
            cinder_config = cinder_config_opt % self.var_puppet_gen

        # collect commands output if the standalone, wsgi or container is up
        if in_ps or in_container:
            self.add_cmd_output(
                "cinder-manage " + cinder_config + " db version",
                suggest_filename="cinder_db_version"
            )
            self.add_cmd_output(
                f"cinder-manage {cinder_config} backup list"
            )
            self.add_cmd_output(
                f"cinder-manage {cinder_config} config list"
            )
            self.add_cmd_output(
                f"cinder-manage {cinder_config} host list"
            )
            self.add_cmd_output(
                f"cinder-status {cinder_config} upgrade check"
            )

            vars_all = [p in os.environ for p in [
                        'OS_USERNAME', 'OS_PASSWORD']]

            vars_any = [p in os.environ for p in [
                        'OS_TENANT_NAME', 'OS_PROJECT_NAME']]

            if not (all(vars_all) and any(vars_any)):
                self.soslog.warning("Not all environment variables set. "
                                    "Source the environment file for the user "
                                    "intended to connect to the OpenStack "
                                    "environment.")
            else:
                list_cmds = [
                    "backend pool",
                    "group type",
                    "message",
                    "qos",
                    "service",
                    "type",
                ]

                for cmd in list_cmds:
                    self.add_cmd_output(f"openstack volume {cmd} list")

                list_cmds_projects = [
                    "backup",
                    "group",
                    "group snapshot",
                    "snapshot",
                    "transfer request",
                    "",
                ]

                for cmd in list_cmds_projects:
                    self.add_cmd_output(
                        f"openstack volume {cmd} list --all-projects"
                    )

                # get details for each volume
                cmd = "openstack volume list -f value --all-projects"
                res = self.exec_cmd(cmd)
                if res['status'] == 0:
                    cinder_volumes = res['output']
                    for volume in cinder_volumes.splitlines():
                        volume = volume.split()[0]
                        cmd = f"openstack volume show {volume}"
                        self.add_cmd_output(cmd)

        self.add_forbidden_path('/etc/cinder/volumes')
        self.add_copy_spec([
            "/etc/cinder/",
            self.var_puppet_gen + "/etc/cinder/",
            self.var_puppet_gen + "/etc/httpd/conf/",
            self.var_puppet_gen + "/etc/httpd/conf.d/",
            self.var_puppet_gen + "/etc/httpd/conf.modules.d/*.conf",
            self.var_puppet_gen + "/etc/my.cnf.d/tripleo.cnf",
            self.var_puppet_gen + "/etc/sysconfig/",
        ])

        if self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/log/cinder/",
                f"/var/log/{self.apachepkg}*/cinder*",
            ])
        else:
            self.add_copy_spec([
                "/var/log/cinder/*.log",
                f"/var/log/{self.apachepkg}*/cinder*.log",
            ])

    def apply_regex_sub(self, regexp, subst):
        """ Apply regex substitution """
        self.do_path_regex_sub("/etc/cinder/*", regexp, subst)
        self.do_path_regex_sub(
            self.var_puppet_gen + "/etc/cinder/*",
            regexp, subst
        )

    def postproc(self):
        protect_keys = [
            "admin_password", "backup_tsm_password", "chap_password",
            "nas_password", "cisco_fc_fabric_password", "coraid_password",
            "eqlx_chap_password", "fc_fabric_password",
            "hitachi_auth_password", "hitachi_horcm_password",
            "hp3par_password", "hplefthand_password", "memcache_secret_key",
            "netapp_password", "netapp_sa_password", "nexenta_password",
            "password", "qpid_password", "rabbit_password", "san_password",
            "ssl_key_password", "vmware_host_password", "zadara_password",
            "zfssa_initiator_password", "hmac_keys", "zfssa_target_password",
            "os_privileged_user_password", "transport_url"
        ]
        connection_keys = ["connection"]

        join_con_keys = "|".join(connection_keys)

        self.apply_regex_sub(
            fr"(^\s*({'|'.join(protect_keys)})\s*=\s*)(.*)",
            r"\1*********"
        )
        self.apply_regex_sub(
            fr"(^\s*({join_con_keys})\s*=\s*(.*)://(\w*):)(.*)(@(.*))",
            r"\1*********\6"
        )


class DebianCinder(OpenStackCinder, DebianPlugin, UbuntuPlugin):

    cinder = False
    apachepkg = 'apache2'
    packages = (
        'cinder-api',
        'cinder-backup',
        'cinder-common',
        'cinder-scheduler',
        'cinder-volume',
        'python-cinder',
        'python3-cinder',
    )


class RedHatCinder(OpenStackCinder, RedHatPlugin):

    cinder = False
    apachepkg = 'httpd'
    packages = ('openstack-selinux',)

    def setup(self):
        super().setup()
        self.add_copy_spec(["/etc/sudoers.d/cinder"])


# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import re

from sos.report.plugins import Plugin, RedHatPlugin, UbuntuPlugin


class CephMON(Plugin, RedHatPlugin, UbuntuPlugin):
    """
    This plugin serves to collect information on monitor nodes within a Ceph
    or microceph cluster. It is designed to collect from several versions of
    Ceph, including versions that serve as the basis for RHCS 4 and RHCS 5.

    Older versions of Ceph will have collections from locations such as
    /var/log/ceph, whereas newer versions (as of this plugin's latest update)
    will have collections from /var/log/ceph/<fsid>/. This plugin attempts to
    account for this where possible across the host's filesystem.

    Users may expect to see several collections twice - once in standard output
    from the `ceph` command, and again in JSON format. The latter of which will
    be placed in the `json_output/` subdirectory within this plugin's directory
    in the report archive. These JSON formatted collections are intended to
    aid in automated analysis.
    """

    short_desc = 'CEPH mon'

    plugin_name = 'ceph_mon'
    profiles = ('storage', 'virt', 'container', 'ceph')
    # note: for RHCS 5 / Ceph v16 the containers serve as an enablement trigger
    # but by default they are not capable of running various ceph commands in
    # this plugin - the `ceph` binary is functional directly on the host
    containers = ('ceph-(.*-)?mon.*',)
    files = ('/var/lib/ceph/mon/*', '/var/lib/ceph/*/mon*',
             '/var/snap/microceph/common/data/mon/*')
    ceph_version = 0

    def setup(self):
        all_logs = self.get_option("all_logs")
        self.ceph_version = self.get_ceph_version()

        microceph_pkg = self.policy.package_manager.pkg_by_name('microceph')
        if not microceph_pkg:
            self.add_file_tags({
                '.*/ceph.conf': 'ceph_conf',
                "/var/log/ceph/(.*/)?ceph-.*mon.*.log": 'ceph_mon_log'
            })

            self.add_forbidden_path([
                "/etc/ceph/*keyring*",
                "/var/lib/ceph/**/*keyring*",
                # Excludes temporary ceph-osd mount location like
                # /var/lib/ceph/tmp/mnt.XXXX from sos collection.
                "/var/lib/ceph/**/tmp/*mnt*",
                "/etc/ceph/*bindpass*"
            ])

            if not all_logs:
                self.add_copy_spec([
                    "/var/log/ceph/**/*ceph-mon*.log"
                ])
            else:
                self.add_copy_spec([
                    "/var/log/ceph/**/*ceph-mon*.log*"
                ])

            self.add_copy_spec([
                "/run/ceph/**/ceph-mon*",
                "/var/lib/ceph/**/kv_backend",
            ])

        else:
            self.add_forbidden_path([
                "/var/snap/microceph/common/**/*keyring*",
                "/var/snap/microceph/current/**/*keyring*",
                "/var/snap/microceph/common/data/mon/*/store.db",
                "/var/snap/microceph/common/state/*",
            ])

            if not all_logs:
                self.add_copy_spec([
                    "/var/snap/microceph/common/logs/*ceph-mon*.log",
                ])
            else:
                self.add_copy_spec([
                    "/var/snap/microceph/common/logs/*ceph-mon*.log*",
                ])

            self.add_copy_spec([
                "/var/snap/microceph/common/data/mon/*",
                "/var/snap/microceph/current/conf/*",
            ])

        self.add_cmd_output("ceph report", tags="ceph_report", stderr=False)
        self.add_cmd_output([
            # The ceph_mon plugin will collect all the "ceph ..." commands
            # which typically require the keyring.

            "ceph config generate-minimal-conf",
            "ceph config log",
            "ceph config-key dump",
            "ceph crash ls",
            "ceph crash stat",
            "ceph features",
            "ceph health detail",
            "ceph insights",
            "ceph log last 10000 debug audit",
            "ceph log last 10000 debug cluster",
            "ceph mgr dump",
            "ceph mgr metadata",
            "ceph mgr module ls",
            "ceph mgr services",
            "ceph mgr versions",
            "ceph mon stat",
            "ceph mon features ls",
            "ceph node ls",
            "ceph osd crush class ls",
            "ceph osd crush dump",
            "ceph osd crush rule ls",
            "ceph osd crush show-tunables",
            "ceph osd crush tree --show-shadow",
            "ceph osd erasure-code-profile ls",
            "ceph osd metadata",
            "ceph osd utilization",
            "ceph telemetry channel ls",
            "ceph telemetry collection ls",
            "ceph telemetry ls",
            "ceph quorum_status",
            "ceph versions",
            "ceph-disk list",
        ])

        crashes = self.collect_cmd_output('ceph crash ls')
        if crashes['status'] == 0:
            for crashln in crashes['output'].splitlines():
                if crashln.endswith('*'):
                    cid = crashln.split()[0]
                    self.add_cmd_output(f"ceph crash info {cid}")

        ceph_cmds = [
            "config dump",
            "device ls",
            "df detail",
            "df",
            "fs dump",
            "fs ls",
            "fs status",
            "mds stat",
            "mon dump",
            "osd blocked-by",
            "osd blocklist ls",
            "osd df tree",
            "osd df",
            "osd dump",
            "osd numa-status",
            "osd perf",
            "osd pool autoscale-status",
            "osd pool ls detail",
            "osd stat",
            "pg dump",
            "pg stat",
            "status",
            "time-sync-status",
        ]

        self.add_cmd_output("ceph health detail --format json-pretty",
                            subdir="json_output",
                            tags="ceph_health_detail")
        self.add_cmd_output("ceph osd tree --format json-pretty",
                            subdir="json_output",
                            tags="ceph_osd_tree")
        self.add_cmd_output(
            [f"ceph tell mon.{mid} mon_status" for mid in self.get_ceph_ids()],
            subdir="json_output",
        )

        self.add_cmd_output([f"ceph {cmd}" for cmd in ceph_cmds])

        # get ceph_cmds again as json for easier automation parsing
        self.add_cmd_output(
            [f"ceph {cmd} --format json-pretty" for cmd in ceph_cmds],
            subdir="json_output",
            stderr=False
        )

    def get_ceph_version(self):
        """ Get the versions of running daemons """

        ver = self.exec_cmd('ceph --version')
        if ver['status'] == 0:
            try:
                _ver = ver['output'].split()[2]
                return int(_ver.split('.')[0])
            except Exception as err:  # pylint: disable=broad-except
                self._log_debug(f"Could not determine ceph version: {err}")
        self._log_error(
            'Failed to find ceph version, command collection will be limited'
        )
        return 0

    def get_ceph_ids(self):
        """ Get the IDs of the Ceph daemons """

        ceph_ids = []
        # ceph version 14 correlates to RHCS 4
        if self.ceph_version in (14, 15):
            # Get the ceph user processes
            out = self.exec_cmd('ps -u ceph -o args')

            if out['status'] == 0:
                # Extract the mon ids
                for procs in out['output'].splitlines():
                    proc = procs.split()
                    # Locate the '--id' value of the process
                    if proc and proc[0].endswith("ceph-mon"):
                        try:
                            id_index = proc.index("--id")
                            ceph_ids.append(proc[id_index + 1])
                        except (IndexError, ValueError):
                            self._log_warn('Unable to find ceph IDs')

        # ceph version 16 is RHCS 5
        elif self.ceph_version >= 16:
            stats = self.exec_cmd('ceph status')
            if stats['status'] == 0:
                try:
                    ret = re.search(r'(\s*mon: .* quorum) (.*) (\(.*\))',
                                    stats['output'])
                    ceph_ids.extend(ret.groups()[1].split(','))
                except Exception as err:  # pylint: disable=broad-except
                    self._log_debug(f"id determination failed: {err}")
        return ceph_ids

    def postproc(self):

        if self.ceph_version >= 16:
            keys = [
                'key',
                'username',
                'password',
                '_secret',
                'rbd/mirror/peer/.*'
            ]
            # we need to do this iteratively, as config-key dump here contains
            # nested json data written as strings, which may have multiple hits
            # within the same line
            for key in keys:
                creg = fr'(((.*)({key}\\\": ))((\\\"(.*?)\\\")(.*)))'
                self.do_cmd_output_sub(
                        'ceph config-key dump', creg, r'\2\"******\"\8'
                )
        else:
            keys = [
                'API_PASSWORD',
                'API_USER.*',
                'API_.*_KEY',
                'key',
                '_secret',
                'rbd/mirror/peer/.*'
            ]

            creg = fr"((\".*({'|'.join(keys)})\":) \")(.*)(\".*)"
            self.do_cmd_output_sub(
                    'ceph config-key dump', creg, r'\1*******\5'
            )

        self.do_cmd_private_sub('ceph config-key dump')

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Fail2Ban(Plugin, IndependentPlugin):

    short_desc = "Fail2Ban daemon"

    plugin_name = "fail2ban"
    profiles = ('security',)
    packages = ('fail2ban', 'fail2ban-server')
    servicess = ('fail2ban',)

    def setup(self):

        self.add_copy_spec([
            '/etc/fail2ban',
        ])

        self.add_cmd_output([
            'fail2ban-client status',
            'fail2ban-client banned',
        ])



# Copyright (C) 2007 Shijoe George <spanjikk@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin, PluginOpt


class TftpServer(Plugin, IndependentPlugin):

    short_desc = 'TFTP Server information'
    plugin_name = 'tftpserver'
    profiles = ('sysmgmt', 'network')
    services = ('tftp', 'tftpd-hpa')
    files = ('/etc/xinetd.d/tftp',)
    packages = ('tftp-server', 'tftpd-hpa')
    option_list = [
        PluginOpt('tftpboot', default=False,
                  desc='collect content from tftpboot path')
    ]

    def setup(self):
        self.add_copy_spec('/etc/default/tftp-hpa')

        tftp_dirs = [
            '/srv/tftp',
            '/tftpboot',
            '/var/lib/tftpboot',
        ]
        self.add_dir_listing(tftp_dirs, recursive=True)

        if self.get_option('tftpboot'):
            self.add_copy_spec(tftp_dirs)

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class Soundcard(Plugin, DebianPlugin, UbuntuPlugin):

    short_desc = 'Sound devices'

    plugin_name = "soundcard"
    profiles = ('desktop', 'hardware')

    def setup(self):
        self.add_copy_spec("/proc/asound/*")
        self.add_cmd_output([
            "aplay -l",
            "aplay -L",
            "amixer"
        ])


class RedHatSoundcard(Soundcard, RedHatPlugin):

    def setup(self):
        super().setup()
        self.add_copy_spec([
            "/etc/alsa/*",
            "/etc/asound.*"
        ])

# vim: set et ts=4 sw=4 :



# Copyright (c) 2024 Lucas Benedito <lbenedit@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class AAPGatewayPlugin(Plugin, RedHatPlugin):
    short_desc = 'AAP Gateway plugin'
    plugin_name = 'aap_gateway'
    profiles = ('sysmgmt', 'ansible',)

    packages = ('automation-gateway',
                'automation-gateway-config')
    commands = ('aap-gateway-manage',)
    services = ('automation-gateway',)

    def setup(self):
        self.add_copy_spec([
            "/var/log/supervisor",
            "/etc/ansible-automation-platform",
            "/etc/supervisord.d/",
            "/var/log/ansible-automation-platform/gateway/",
        ])

        self.add_forbidden_path([
            "/etc/ansible-automation-platform/gateway/SECRET_KEY",
            "/etc/ansible-automation-platform/gateway/*.key",
            "/etc/ansible-automation-platform/gateway/*.cert",
        ])

        self.add_cmd_output("aap-gateway-manage list_services")
        self.add_dir_listing("/etc/ansible-automation-platform/",
                             recursive=True)

    def postproc(self):
        # remove database password
        jreg = r"(DATABASE_PASSWORD)(\s*)(=|:)(\s*)(.*)"
        repl = r"\1\2\3\4********"
        self.do_path_regex_sub(
            "/etc/ansible-automation-platform/gateway/settings.py",
            jreg,
            repl)

# vim: set et ts=4 sw=4 :



# Copyright (C) 2015 Red Hat, Inc., Lee Yarwood <lyarwood@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.


from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class OpenStackTrove(Plugin):

    short_desc = 'OpenStack Trove'

    plugin_name = "openstack_trove"
    profiles = ('openstack', 'openstack_controller')
    var_puppet_gen = "/var/lib/config-data/puppet-generated/trove"

    def setup(self):
        if self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/log/trove/",
            ])
        else:
            self.add_copy_spec([
                "/var/log/trove/*.log",
            ])

        self.add_copy_spec([
            '/etc/trove/',
            self.var_puppet_gen + '/etc/trove/'
        ])

    def apply_regex_sub(self, regexp, subst):
        """ Apply regex substitution """
        self.do_path_regex_sub("/etc/trove/*", regexp, subst)
        self.do_path_regex_sub(
            self.var_puppet_gen + "/etc/trove/*",
            regexp, subst
        )

    def postproc(self):
        protect_keys = [
            "default_password_length", "notifier_queue_password",
            "rabbit_password", "replication_password", "admin_password",
            "dns_passkey", "transport_url", "memcache_secret_key"
        ]
        connection_keys = ["connection"]

        join_con_keys = "|".join(connection_keys)

        self.apply_regex_sub(
            fr"(^\s*({'|'.join(protect_keys)})\s*=\s*)(.*)",
            r"\1*********"
        )
        self.apply_regex_sub(
            fr"(^\s*({join_con_keys})\s*=\s*(.*)://(\w*):)(.*)(@(.*))",
            r"\1*********\6"
        )


class DebianTrove(OpenStackTrove, DebianPlugin, UbuntuPlugin):

    packages = (
        'python-trove',
        'trove-common',
        'trove-api',
        'trove-taskmanager',
        'python3-trove',
    )


class RedHatTrove(OpenStackTrove, RedHatPlugin):

    packages = ('openstack-selinux',)

# vim: set et ts=4 sw=4 :



# Copyright (C) 2018 Red Hat, Inc., Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from re import match
from shlex import quote
from sos.report.plugins import Plugin, RedHatPlugin


class Candlepin(Plugin, RedHatPlugin):

    short_desc = 'Candlepin entitlement management'

    plugin_name = 'candlepin'
    packages = ('candlepin',)

    dbhost = None
    dbpasswd = None
    env = None

    def setup(self):
        # for external DB, search in /etc/candlepin/candlepin.conf for:
        # org.quartz.dataSource.myDS.URL=..
        #
        # and for DB password, search for
        # org.quartz.dataSource.myDS.password=..
        self.dbhost = "localhost"
        self.dbpasswd = ""
        cfg_file = "/etc/candlepin/candlepin.conf"
        try:
            with open(cfg_file, 'r', encoding='UTF--8') as cfile:
                candle_lines = cfile.read().splitlines()
            for line in candle_lines:
                # skip empty lines and lines with comments
                if not line or line[0] == '#':
                    continue
                if match(r"^\s*org.quartz.dataSource.myDS.URL=\S+", line):
                    self.dbhost = line.split('=')[1]
                    # separate hostname from value like
                    # jdbc:postgresql://localhost:5432/candlepin
                    self.dbhost = self.dbhost.split('/')[2].split(':')[0]
                if match(r"^\s*org.quartz.dataSource.myDS.password=\S+", line):
                    self.dbpasswd = line.split('=')[1]
        except (IOError, IndexError):
            # fallback when the cfg file is not accessible or parseable
            pass

        self.add_file_tags({
            '/var/log/candlepin/candlepin.log.*': 'candlepin_log',
            '/var/log/candlepin/err.log.*': 'candlepin_error_log',
            '/etc/candlepin/candlepin.conf': 'candlepin_conf'
        })

        # set the password to os.environ when calling psql commands to prevent
        # printing it in sos logs
        # we can't set os.environ directly now: other plugins can overwrite it
        self.env = {"PGPASSWORD": self.dbpasswd}

        # Always collect the full active log of these
        self.add_copy_spec([
            "/var/log/candlepin/error.log",
            "/var/log/candlepin/candlepin.log"
        ], sizelimit=0)

        # Allow limiting on logrotated logs
        self.add_copy_spec([
            "/etc/candlepin/candlepin.conf",
            "/etc/candlepin/broker.xml",
            "/var/log/candlepin/audit*.log*",
            "/var/log/candlepin/candlepin.log[.-]*",
            "/var/log/candlepin/cpdb*.log*",
            "/var/log/candlepin/cpinit*.log*",
            "/var/log/candlepin/error.log[.-]*",
            # Specific to candlepin, ALL catalina logs are relevant. Adding it
            # here rather than the tomcat plugin to ease maintenance and not
            # pollute non-candlepin sos reports that enable the tomcat plugin
            "/var/log/tomcat*/catalina*log*",
            "/var/log/tomcat*/host-manager*log*",
            "/var/log/tomcat*/localhost*log*",
            "/var/log/tomcat*/manager*log*",
        ])

        self.add_cmd_output("du -sh /var/lib/candlepin/*/*")
        # collect tables sizes, ordered
        _cmd = self.build_query_cmd(
            "SELECT table_name, pg_size_pretty(total_bytes) AS total, "
            "pg_size_pretty(index_bytes) AS INDEX , "
            "pg_size_pretty(toast_bytes) AS toast, pg_size_pretty(table_bytes)"
            " AS TABLE FROM ( SELECT *, "
            "total_bytes-index_bytes-COALESCE(toast_bytes,0) AS table_bytes "
            "FROM (SELECT c.oid,nspname AS table_schema, relname AS "
            "TABLE_NAME, c.reltuples AS row_estimate, "
            "pg_total_relation_size(c.oid) AS total_bytes, "
            "pg_indexes_size(c.oid) AS index_bytes, "
            "pg_total_relation_size(reltoastrelid) AS toast_bytes "
            "FROM pg_class c LEFT JOIN pg_namespace n ON "
            "n.oid = c.relnamespace WHERE relkind = 'r') a) a order by "
            "total_bytes DESC"
        )
        self.add_cmd_output(_cmd, suggest_filename='candlepin_db_tables_sizes',
                            env=self.env)

        _cmd = self.build_query_cmd("\
            SELECT displayname, content_access_mode \
            FROM cp_owner;")
        self.add_cmd_output(_cmd,
                            suggest_filename='simple_content_access',
                            env=self.env)

    def build_query_cmd(self, query, csv=False):
        """
        Builds the command needed to invoke the pgsql query as the postgres
        user.
        The query requires significant quoting work to satisfy both the
        shell and postgres parsing requirements. Note that this will generate
        a large amount of quoting in sos logs referencing the command being run
        """
        csvformat = "-A -F , -X" if csv else ""
        _dbcmd = "psql --no-password -h %s -p 5432 -U candlepin \
                  -d candlepin %s -c %s"
        return _dbcmd % (self.dbhost, csvformat, quote(query))

    def postproc(self):
        reg = r"(((.*)(pass|token|secret)(.*))=)(.*)"
        repl = r"\1********"
        self.do_file_sub("/etc/candlepin/candlepin.conf", reg, repl)
        cpdbreg = r"(--password=)([a-zA-Z0-9]*)"
        self.do_file_sub("/var/log/candlepin/cpdb.log", cpdbreg, repl)
        for key in ["trustStorePassword", "keyStorePassword"]:
            self.do_file_sub("/etc/candlepin/broker.xml",
                             fr"({key})=(\w*)([;<])",
                             r"\1=********\3")

# vim: set et ts=4 sw=4 :



# Copyright Red Hat 2020, Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import re

from xml.etree import ElementTree
from sos.collector.clusters import Cluster
from sos.utilities import sos_parse_version


class pacemaker(Cluster):

    cluster_name = 'Pacemaker High Availability Cluster Manager'
    sos_plugins = ['pacemaker']
    packages = ('pacemaker',)
    strict_node_list = True
    option_list = [
        ('online', True, 'Collect nodes listed as online'),
        ('offline', True, 'Collect nodes listed as offline'),
        ('only-corosync', False, 'Only use corosync.conf to enumerate nodes')
    ]

    def get_nodes(self):
        self.nodes = []
        # try crm_mon first
        try:
            if not self.get_option('only-corosync'):
                try:
                    self.get_nodes_from_crm()
                except Exception as err:
                    self.log_warn("Falling back to sourcing corosync.conf. "
                                  f"Could not parse crm_mon output: {err}")
            if not self.nodes:
                # fallback to corosync.conf, in case the node we're inspecting
                # is offline from the cluster
                self.get_nodes_from_corosync()
        except Exception as err:
            self.log_error(f"Could not determine nodes from cluster: {err}")

        _shorts = [n for n in self.nodes if '.' not in n]
        if _shorts:
            self.log_warn(
                f"WARNING: Node addresses '{','.join(_shorts)}' may not "
                "resolve locally if you are not running on a node in the "
                "cluster. Try using option '-c pacemaker.only-corosync' if "
                "these connections fail."
            )
        return self.nodes

    def get_nodes_from_crm(self):
        """
        Try to parse crm_mon output for node list and status.
        """
        xmlopt = '--output-as=xml'
        # older pacemaker had a different option for xml output
        _ver = self.exec_primary_cmd('crm_mon --version')
        if _ver['status'] == 0:
            cver = _ver['output'].split()[1].split('-')[0]
            if sos_parse_version(cver) <= sos_parse_version('2.0.3'):
                xmlopt = '--as-xml'
        else:
            return
        _out = self.exec_primary_cmd(
            f"crm_mon --one-shot --inactive {xmlopt}",
            need_root=True
        )
        if _out['status'] == 0:
            self.parse_crm_xml(_out['output'])

    def parse_crm_xml(self, xmlstring):
        """
        Parse the xml output string provided by crm_mon
        """
        _xml = ElementTree.fromstring(xmlstring)
        nodes = _xml.find('nodes')
        for node in nodes:
            _node = node.attrib
            if self.get_option('online') and _node['online'] == 'true':
                self.nodes.append(_node['name'])
            elif self.get_option('offline') and _node['online'] == 'false':
                self.nodes.append(_node['name'])

    def get_nodes_from_corosync(self):
        """
        As a fallback measure, read corosync.conf to get the node list. Note
        that this prevents us from separating online nodes from offline nodes.
        """
        self.log_warn("WARNING: unable to distinguish online nodes from "
                      "offline nodes when sourcing from corosync.conf")
        cc = self.primary.read_file('/etc/corosync/corosync.conf')
        nodes = re.findall(r'((\sring0_addr:)(.*))', cc)
        for node in nodes:
            self.nodes.append(node[-1].strip())

# vim: set et ts=4 sw=4 :



# Copyright (c) 2023 Canonical Ltd., Chi Wai Chan <chiwai.chan@canonical.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.


import subprocess

from sos.collector.exceptions import JujuNotInstalledException
from sos.collector.transports import RemoteTransport
from sos.utilities import sos_get_command_output


class JujuSSH(RemoteTransport):
    """
    A "transport" that leverages `juju ssh` to perform commands on the remote
    hosts.

    This transport is expected to be used in juju managed environment, and the
    user should have the necessary credential for accessing the controller.
    When using this transport, the --nodes option will be expected to be a
    comma separated machine IDs, **not** IP addr, since `juju ssh` identifies
    the ssh target by machine ID.

    Examples:

    sos collect --nodes 0,1,2 --no-local --transport juju --batch

    """

    name = "juju_ssh"
    default_user = "ubuntu"

    def _check_juju_installed(self):
        cmd = "juju version"
        try:
            subprocess.check_output(cmd, stderr=subprocess.STDOUT, shell=True)
        except subprocess.CalledProcessError as err:
            self.log_error("Failed to check `juju` version")
            raise JujuNotInstalledException from err
        return True

    def _chmod(self, fname):
        cmd = f"{self.remote_exec} sudo chmod o+r {fname}"
        try:
            subprocess.check_output(cmd, stderr=subprocess.STDOUT, shell=True)
        except subprocess.CalledProcessError:
            self.log_error(f"Failed to make {fname} world-readable")
            raise
        return True

    def _connect(self, password=""):
        self._connected = self._check_juju_installed()
        return self._connected

    def _disconnect(self):
        return True

    @property
    def connected(self):
        return self._connected

    @property
    def remote_exec(self):
        model, target_option = self.address.split(":")
        model_option = f"-m {model}" if model else ""
        option = f"{model_option} {target_option}"
        return f"juju ssh {option}"

    def _copy_file_to_remote(self, fname, dest):
        model, unit = self.address.split(":")
        model_option = f"-m {model}" if model else ""
        cmd = f"juju scp {model_option} -- {fname} {unit}:{dest}"
        res = sos_get_command_output(cmd, timeout=15)
        return res["status"] == 0

    def _retrieve_file(self, fname, dest):
        self._chmod(fname)  # juju scp needs the archive to be world-readable
        model, unit = self.address.split(":")
        model_option = f"-m {model}" if model else ""
        cmd = f"juju scp {model_option} -- -r {unit}:{fname} {dest}"
        res = sos_get_command_output(cmd)
        return res["status"] == 0


# vim: set et ts=4 sw=4 :



# Copyright (C) 2018 Red Hat, Inc. Daniel Walsh <dwalsh@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import (Plugin, RedHatPlugin, UbuntuPlugin,
                                SoSPredicate, PluginOpt, CosPlugin)


class CRIO(Plugin, RedHatPlugin, UbuntuPlugin, CosPlugin):

    short_desc = 'CRI-O containers'
    plugin_name = 'crio'
    profiles = ('container',)
    packages = ('cri-o', 'cri-tools')
    services = ('crio',)

    option_list = [
        PluginOpt('all', default=False,
                  desc='collect for all containers, even terminated ones'),
        PluginOpt('logs', default=False,
                  desc='collect stdout/stderr logs for containers')
    ]

    def setup(self):
        self.add_copy_spec([
            "/etc/containers",
            "/etc/crictl.yaml",
            "/etc/crio/crio.conf",
            "/etc/crio/seccomp.json",
            "/etc/crio/crio.conf.d/",
            "/etc/systemd/system/cri-o.service",
            "/etc/sysconfig/crio-*"
        ])

        self.add_cmd_output([
            "crio config"
        ])

        self.add_dir_listing('/etc/cni', recursive=True)

        # base cri-o installation does not require cri-tools, which is what
        # supplies the crictl utility
        self.set_cmd_predicate(SoSPredicate(self, packages=['cri-tools']))

        subcmds = [
            'info',
            'images',
            'pods',
            'ps',
            'ps -a',
            'ps -va',
            'stats',
            'version',
        ]

        self.add_cmd_output([f"crictl {s}" for s in subcmds])

        ps_cmd = 'crictl ps --quiet'
        if self.get_option('all'):
            ps_cmd = f"{ps_cmd} -a"

        img_cmd = 'crictl images --quiet'
        pod_cmd = 'crictl pods --quiet'

        containers = self._get_crio_list(ps_cmd)
        images = self._get_crio_list(img_cmd)
        pods = self._get_crio_list(pod_cmd)

        self._get_crio_goroutine_stacks()

        for container in containers:
            self.add_cmd_output(f"crictl inspect {container}",
                                subdir="containers")
            if self.get_option('logs'):
                self.add_cmd_output(f"crictl logs -t {container}",
                                    subdir="containers/logs", priority=100,
                                    tags="crictl_logs")

        for image in images:
            self.add_cmd_output(f"crictl inspecti {image}", subdir="images")

        for pod in pods:
            self.add_cmd_output(f"crictl inspectp {pod}", subdir="pods")

    def _get_crio_list(self, cmd):
        ret = []
        result = self.exec_cmd(cmd)
        if result['status'] == 0:
            for ent in result['output'].splitlines():
                ret.append(ent)
            # Prevent the socket deprecation warning from being iterated over
            if ret and 'deprecated' in ret[0]:
                ret.pop(0)
        return ret

    def _get_crio_goroutine_stacks(self):
        if self.signal_process_usr1(r'^/usr/bin/crio$'):
            self.add_copy_spec("/tmp/crio-goroutine-stacks*.log")

# vim: set et ts=4 sw=4 :



# Copyright (C) 2024 Red Hat, Inc., Jose Castillo <jcastillo@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import pwd
from sos.report.plugins import Plugin, IndependentPlugin, PluginOpt


class Instructlab(Plugin, IndependentPlugin):
    """
    This plugin is used to capture information about
    Instructlab installations.
    InstructLab is an open source project for enhancing
    large language models (LLMs) used in generative
    artificial intelligence (gen AI) applications.
    Instructlab can run either as a container, or directly
    outside a container.
    """

    short_desc = 'Instructlab'
    plugin_name = 'instructlab'
    profiles = ('ai',)
    containers = ('instructlab', 'ilab',)
    commands = ('ilab',)

    option_list = [
        PluginOpt('ilab-user', default='cloud-user', val_type=str,
                  desc='user that runs instructlab'),
        PluginOpt('ilab-conf-dir', default='', val_type=str,
                  desc='instructlab data directory'),
        PluginOpt('get-cache', default=False,
                  desc='Capture models and osci cached data')
    ]

    def setup(self):
        cont_share_conf_path = "/usr/share/instructlab/config"
        cont_opt_path = "/opt/app-root/src"
        # .cache dir contains the models and oci directories
        # which can be quite big. We'll gather this only if
        # specifying it via command line option
        cache_dir = ".cache/instructlab"
        # .config is where the configuration yaml files can
        # be found. We gather this always.
        config_dir = ".config/instructlab"
        # In the .local directory we can find datasets,
        # chat logs, taxonomies, and other very useful data
        # We gather this always.
        local_share_dir = ".local/share/instructlab"

        # container paths
        cont_cache_path = self.path_join(cont_opt_path, cache_dir)
        cont_config_path = self.path_join(cont_opt_path, config_dir)
        cont_local_path = self.path_join(cont_opt_path, local_share_dir)

        self.add_forbidden_path([
            self.path_join(cont_local_path,
                           "taxonomy/.git"),
            self.path_join(cont_local_path, "taxonomy/.github"),
            self.path_join(cont_opt_path,
                           "src/.local/share/instructlab/taxonomy/.git"),
            self.path_join(cont_opt_path,
                           "src/.local/share/instructlab/taxonomy/.github"),
        ])

        subcmds = [
            'taxonomy diff',
            'taxonomy diff --taxonomy-base=empty',
            'system info',
            'model list',
            'config show',
        ]

        data_dirs = [
            'data',
            'generated',
            'taxonomy',
            'taxonomy_data',
            'chatlogs',
            'checkpoints',
            'datasets',
            'internal',
            'phased',
        ]

        # If containerized, run commands in containers
        try:
            ilab_con = self.get_all_containers_by_regex("instructlab*")[0][1]
        except Exception:  # pylint: disable=broad-except
            ilab_con = None

        self.add_copy_spec(
            [f"{cont_share_conf_path}/rhel_ai_config.yaml",
             f"{cont_config_path}/config.yaml"],
            container=ilab_con
        )
        self.add_copy_spec(
            [self.path_join(cont_local_path, data_dir)
             for data_dir in data_dirs],
            container=ilab_con
        )
        self.add_cmd_output(
            [f"ilab {sub}" for sub in subcmds],
            container=ilab_con
        )
        self.add_dir_listing(
            cont_cache_path,
            recursive=True,
            container=ilab_con
        )
        if self.get_option('get-cache'):
            self.add_copy_spec(
                f"{cont_cache_path}",
                container=ilab_con
            )
        self.add_container_logs(list(self.containers))

        ilab_user = self.get_option("ilab-user")
        try:
            user_pwd = pwd.getpwnam(ilab_user)
        except KeyError:
            # The user doesn't exist, this will skip the rest
            self._log_warn(
                f'User "{ilab_user}" does not exist, will not '
                'collect Instructlab information. Use '
                '`-k instructlab.ilab-user` option to define '
                'the user to use to collect data for Instructlab')
            return

        if user_pwd:
            ilab_dir = user_pwd.pw_dir
            if self.get_option("ilab-conf-dir"):
                ilab_dir = self.path_join(
                    ilab_dir, self.get_option('ilab-conf-dir')
                )
            data_dirs_base = self.path_join(ilab_dir, local_share_dir)

            self.add_copy_spec(self.path_join(ilab_dir, config_dir))
            self.add_copy_spec([
                self.path_join(data_dirs_base, data_dir)
                for data_dir in data_dirs
            ])
            self.add_dir_listing(
                self.path_join(ilab_dir, cache_dir),
                recursive=True
            )

            if self.get_option("get-cache"):
                self.add_copy_spec(
                    self.path_join(ilab_dir, cache_dir)
                )

# vim: set et ts=4 sw=4 :
